"""Contains default systems that can be used in simulations
"""
# Copyright should be placed above imports
#pylint: disable=wrong-import-position
__copyright__ = 'Copyright (c) 2014-2016 SSPA Sweden AB'

#The pylint warning is temporarily disabled here, should be enabled once the
#structure for build in systems is in place
#pylint: disable=wildcard-import
from pysim.systems.defaultsystemcollection1 import *
